<?php
  /**
   * The Main controller is the default controller of the framework, All functions except for the __construct can be removed
   *
   * @author Macdonald Terrence Robinson
   * @package Core
   *
   */
  require_once 'BaseController.php';
  
  class Main extends BaseController
  {
    public function __construct()
    {
      parent::__construct();      
    }

    public function test()
    {
    }

    public function sockets_test()
    {
      require_once 'libs/standalone/Sockets.php';

      $host = 'www.google.ca';
      $path = '/';
      $params = array();

      list($header, $return) = Sockets::httpRequest($host, $path, $params);
      print_r($return);
    }

    public function zip_test()
    {
      //First we include the ZIP class file
      require_once 'libs/standalone/ZIP.php';

      //This is the directory to ZIP
      $dirZip = 'writable/chat';

      //This is the path of where the ZIP will be created
      $zipPath = 'writable/zip-test.zip';

      //If there is already a ZIP at that path, delete it
      if(file_exists($zipPath))
        unlink($zipPath);

      //Create an instance of the ZIP class passing in the ZIP path
      $zipHandler =  new ZIP($zipPath);

      //We then add the directory to ZIP
      //View the API documentation at:
      //http://www.phpconcept.net/pclzip/man/en/index.php?methods-add
      $zipHandler->add($dirZip
                ,PCLZIP_OPT_REMOVE_PATH, Config::$siteDir
                ,PCLZIP_OPT_ADD_PATH, "Logs Backup");

      //This is a Utility function that is built into the framework
      //It forces the download of a file
      Utility::forceDownload($zipPath);

    }

    public function fileUpload()
    {
      print_r($_FILES);
      if( $this->HasFiles() )
      {
        require_once 'libs/standalone/FileUpload.php';
        $fileUpload = new FileUpload('uploadFile');
        $fileUpload->moveAllFilesTo(Config::$siteDir.Config::$coreDir);       
      }
      $this->returnContent( 'Done' );
    }
    
    public function crawler()
    {
      $contentVars = array();
      $contentVars['crawleResults'] = '';

      require_once 'libs/standalone/Crawler.php';
      $crawler = new Crawler(Config::$writableDir.'crawlerLog/crawlerLog.php');

      $contentVars['crawlerDataFile'] = $crawler->getDataFilePath();

      if(( $this->IsPost() ) || ( Get::isExists('pageIndex') ))
      {
        require_once 'libs/standalone/Pagination.php';
        $pagination = new Pagination('crawler', 2);

        if( !Get::isExists('paginationKey') )
        {
          $searchString = '';

          if(Post::isExists('SearchString'))
            $searchString = Post::getByKey('SearchString');


          if($searchString !='')
            $crawler->start(Config::$siteURL, $searchString, true, false);

          $pagination->setResults($crawler->foundLog);

          $results = $pagination->getResultsByPageIndex(0);

          foreach ($results as $key=>$value)
            $contentVars['crawleResults'] .= $crawler->renderResult($key, $value, false);

          $contentVars['crawleResults'] .= $pagination->renderNav(0, 'ajax', '#crawleResults');

          if( count($results) == 0)
            $contentVars['crawleResults'] = 'Sorry no results found';

        }
        else
        {
          if( Get::isExists('pageIndex') )
          {
            $results = $pagination->getResultsByPageIndex( Get::getByKey('pageIndex') );

            foreach ($results as $key=>$value)
              $contentVars['crawleResults'] .= $crawler->renderResult($key, $value, false);

            $contentVars['crawleResults'] .= $pagination->renderNav(Get::getByKey('pageIndex'), 'ajax', '#crawleResults');
          }
        }
      }
            
      $searchField = '<form method="post" action="{siteURL}{currentController}/{currentFunction}" class="ajax" ajaxOutput="#crawleResults" onAjaxWait="Searching please wait...">
        Search String: <input type="text" name="SearchString" />
        <input type="submit" />
      </form>';
      
      $contentVars['crawlerSearchField'] = Parser::parseData($searchField);
      
      $this->returnContent($this->loadMainContent('crawler', $contentVars), $contentVars['crawleResults']);
    }   
    
    public function generateImage()
    {
      require_once 'libs/standalone/GD.php';

      $gd = new GD();

      $image = $gd->loadImage('temp/fantasy-3dart02.jpg');
      $image2 = $gd->loadImage('temp/php.gif');
      $mask = $gd->loadImage('temp/mask.jpg');
      
      $newImage = $image->resize('50%', '50%', 'fill');
      $newImage = $newImage->applyMask($mask, 50, 50);
      $newImage = $newImage->merge($image2, 100, 100, 50);

      $gd->writeTextToImage($newImage, 'Hi :)', 350, 250, '#ff0000', 20, 45, 40);

      header('Content-Type: image/png');
      $this->returnContent($newImage->asString('png'),'',true);
    }
   
    public function pdf()
    {
      if( count($_POST)>0 )
      {
        require_once 'libs/standalone/PDF.php';
        $pdf = new PDF('test.pdf');
        $pdf->generateFromFile(Config::$siteURL.'temp/pdftest.php');
      }
      
      $this->returnContent($this->loadMainContent('pdf'));
    }
           
    public function chat()
    {
      $plugin = $this->loadPlugin('Chat');
      $plugin->start();
    }
    
    private function generatePlayer1()
    {
      $media3 = new Media(Config::$siteURL, 'MyMovie4', '300','100');

      $audioFilePath = Config::$siteURL.'audios/betty.mp3';
      $this->addToFrameworkCode($media3->generateAudioEmbedCode($audioFilePath));

      return $media3->generateDiv();
    }
    
    private function generatePlayer2()
    {
      $media3 = new Media(Config::$siteURL, 'MyMovie3', '400', '300');
      
      $flvFilePath = Config::$siteURL.'videos/3D_wallpapers_video_2.flv';
      $this->addToFrameworkCode($media3->generateFLVEmbedCode($flvFilePath));
      
      return $media3->generateDiv();
    }
    
    public function mediaPlayers()
    {
      require_once 'libs/standalone/Media.php';
      $this->addJSFile('js/swfobject/swfobject.js');
      
      $contentVars['MyMovie1Div'] = $this->generatePlayer1();
      $contentVars['MyMovie2Div'] = $this->generatePlayer2();

      $this->returnContent($this->loadMainContent('media', $contentVars));
    }
    
    public function googleMaps()
    {
      require_once 'libs/standalone/GoogleMaps.php';
      $this->addJSFile('js/jquery.jmap.js');
      $this->addJSFile('js/jMapHelper.js');
      
      $googleMaps = new GoogleMaps('ABQIAAAAS19JrWyNTpsdhmygr-PAPxRPN4WJ3OS1FhR1Fz3VqeggBqTzHBR86OieU5h4P2VUWJVJehyPi8DkNg', 'map');
      $googleMaps->setZoom(2);

      $googleMaps->addLatLong('37.4419', '-122.1419','This is a test');
      $googleMaps->addAddress('Toronto','Toronto');
      $googleMaps->addAddress('Calgary','Calgary');
      
      $this->addToFrameworkCode($googleMaps->generateScript());
      
      $this->returnContent($this->loadMainContent('googleMaps'));
    }
    
    public function geoTargeting()
    {
      $contentVars = array();
      /*require_once 'libs/standalone/IPLookup.php';
      
      $contentVars = array();

      $IPLookup = new IPLookup();

      // To Get a list of all the IP addresses
      $ipAddress1 = $IPLookup->getIPsFromDomainName('google.com');

      // To Get 1 IP address
      $ipAddress2 = $IPLookup->getIPFromDomainName('youtube.com');

      $google =  '<pre>Google<br />
      '.print_r($IPLookup->lookup($ipAddress1[0]), true).'</pre>';
      $contentVars['Google'] = $google;
      
      $yahoo = '<pre>Yahoo<br />'
      .print_r($IPLookup->lookup($ipAddress2), true).'</pre>';
      $contentVars['Yahoo'] = $yahoo;

      $IPLookup->close();*/

      $this->returnContent($this->loadMainContent('geoTargeting', $contentVars));
    }
    
  }
?>
