<?php
  /**
   * This is the Base class that every Plugin must extend
   *
   * @author Macdonald Terrence Robinson
   * @package Core
   *
   */
  class MainPlugin
  {
    protected $controller = NULL;
    protected $callingFuntion;

    protected function __construct(MainController $controller)
    {
      $this->controller = $controller;      
    }

    public function __call($name, $arguments)
    {
      if( Utility::isFunctionInObject($this->controller, $name) )
        return Master::initFunction( $this->controller, $name, $arguments );
    }

    public function getCurrentPluginPath()
    {
      return Config::$pluginsDir.'/'.get_class($this).'/';
    }

    public function loadContent($contentName, $keyValuePair)
    {      
      $dir1 = $this->getCurrentPluginPath().'views/'.Config::$siteLanguage.'/';
      $dir2 = $this->getCurrentPluginPath().'views/common/';

      $path1 = $dir1.$contentName.'.php';
      $path2 = $dir2.$contentName.'.php';

      if( FileSystem::fileExists($path1) )
        return Parser::compileAndParse($path1, $keyValuePair);
      else if( FileSystem::fileExists($path2) )
        return Parser::compileAndParse($path2, $keyValuePair);
      else
        ErrorHandler::runTimeError("Cannot find plugin content: $contentName");
    }

    public function loadModel($modelName)
    {
      $filePath = $this->getCurrentPluginPath().'models/'.$modelName.'.php';
      //return Master::loadAndInitClass($modelName, array(), $filePath);
      return Master::loadAndInitClass($modelName, array(), $filePath);

    }

    public function start()
    {      
      $this->callingFuntion = URI::getSegment(1);
      $function = URI::getSegment(2);

      $params = URI::getParams();
      array_shift($params);

      if( Utility::isFunctionInObject( $this, $function) )
        return Master::initFunction($this, $function, $params);
      else
      {
        $defaultFunction = Config::$defaultFunction;
        if( Utility::isFunctionInObject( $this, $defaultFunction) )
          return Master::initFunction($this, $defaultFunction, $params);
        else
        {
          $errorMessage = 'The default function ( '.$defaultFunction.' ) is not defined for the plugin ( '.get_class($this).' )';
          return ErrorHandler::runTimeError($errorMessage);
        }
      }
    }

  }
?>
