<?php
/**
 * Helps to upload files, also supports multiple files
 *
 * @author Macdonald Robinson
 */
class FileUpload
{
  private $allowedExtentions = array();
  private $errors = array();
  private $success = array();
  private $uploadFieldNameToProcess = '';

  public function __construct( $uploadFieldNameToProcess, $allowedExtentions = array('txt', 'jpg', 'jpeg', 'gif', 'png', 'doc', 'pdf', 'docx') )
  {
    $this->allowedExtentions = $allowedExtentions;
    $this->uploadFieldNameToProcess = $uploadFieldNameToProcess;
  }

  public function getErrors()
  {
    return $this->errors;
  }

  public function getSuccess()
  {
    return $this->success;
  }

  private function setError($errorMessage)
  {
    $this->errors[] = $errorMessage;
  }

  private function setSuccess($message)
  {
    $this->success[] = $message;
  }

  public function moveAllFilesTo($destPath)
  {
    if( !is_dir($destPath) || !file_exists($destPath) )
    {
      $this->setError('Destination directory: ('.$destPath.') does not exist');
      return false;
    }

    $files = $_FILES[$this->uploadFieldNameToProcess];

    if(is_array($files['tmp_name']))
    {
      foreach( $files['tmp_name'] as $key=>$tempPath )
      {
        if($tempPath =='')
          continue;

        $fileName = $this->format($files['name'][$key]);
        $targetPath = $destPath.$fileName;
        $this->moveTempFile($tempPath, $targetPath);
      }
    }
    else if(($files['tmp_name'] !='') && ( is_string($files['tmp_name']) ))
    {
        $fileName = $this->format($files['name']);
        $tempPath = $files['tmp_name'];
        
        $targetPath = $destPath.$fileName;
        $this->moveTempFile($tempPath, $targetPath);
    }
  }

  private function format($str)
  {
    $formated = trim($str);
    $formated = strtolower($formated);
    $formated = strip_tags($formated);
    $formated = htmlentities($formated);
    $formated = str_replace(array(' ','&amp;','\\','/','\'','"','`',';'), array('-','-',''), $formated);

    return trim($formated);
  }

  private function moveTempFile($tempPath, $destPath)
  {
    $segments = explode('.', $destPath);
    $extention = strtolower(trim($segments[count($segments)-1]));

    if(!in_array($extention, $this->allowedExtentions) )
    {
      $this->setError('Invalid file extention: '.$destPath);
      return false;
    }

    $return = move_uploaded_file($tempPath, $destPath);

    if($return)
    {
      $this->setSuccess('Successfully uploaded: '.$destPath);
      return true;
    }
    else
    {
      $this->setError('Error uploading: '.$destPath);
      return false;
    }
  }
}
?>
