<?php
  /**
   * The class provides helper functions to pagenate over any kind of one dimentional array
   *
   * @author Macdonald Terrence Robinson
   * @package Libraries
   *
   */
  class Pagination
  {
    private $paginationKey = '';
    private $resultsPerPage = 5;
    private $currentPageIndex = 0;
    private $maxPaginationTabs = 10;
    
    public function __construct($paginationKey, $resultsPerPage = 5, $maxPaginationTabs = 10)
    {
      $this->paginationKey = $paginationKey;
      $this->resultsPerPage = $resultsPerPage;
      $this->maxPaginationTabs = $maxPaginationTabs;
    }
    
    public function getPrefix()
    {
      return 'Pagination_';
    }

    public function getKey()
    {
      return $this->getPrefix().$this->paginationKey;
    }

    public function setResultsPerPage($resultsPerPage)
    {
      $this->resultsPerPage = $resultsPerPage;
    }
    
    public function setResults(array $result)
    {
      $_SESSION[$this->getKey()] = $result;
    }
    
    public function getTotalNumberofPages()
    {
      $result = count($this->getResults())/$this->resultsPerPage;
      $intResult = intval($result);

      if($result > $intResult)
        $intResult += 1;
      
      return $intResult;
    }
    
    public function getResults()
    {
      return (isset($_SESSION[$this->getKey()]) && is_array($_SESSION[$this->getKey()])) ? $_SESSION[$this->getKey()] : array();
    }
    
    public function getResultsByPageIndex($pageIndex)
    {
      $this->currentPageIndex = $pageIndex;
      $startRecord = $pageIndex * $this->resultsPerPage;
      $results = $this->getResults();
      return array_splice($results, $startRecord, $this->resultsPerPage);
    }

    private function renderFirstAndPrevious($currentPageIndex, $class='', $ajaxOutput='', $urlToPostTo='', $customQueryString='')
    {
      $code = '';
      if($currentPageIndex != 0)
      {
        $code .= '<li><a href="'.$urlToPostTo.'?paginationKey='.$this->paginationKey.'&pageIndex=0&'.$customQueryString.'" class="'.$class.' first" ajaxOutput="'.$ajaxOutput.'"><<</a></li>';
        $code .= '<li><a href="'.$urlToPostTo.'?paginationKey='.$this->paginationKey.'&pageIndex='.($currentPageIndex-1).'&'.$customQueryString.'" class="'.$class.' previous" ajaxOutput="'.$ajaxOutput.'"><</a></li>';
      }
      else
      {
        $code .= '<li class="disabled"><a><<</a></li>';
        $code .= '<li class="disabled"><a><</a></li>';
      }

      return $code;
    }

    private function renderNextAndLast($currentPageIndex, $class='', $ajaxOutput='', $urlToPostTo='', $customQueryString='')
    {
      $code = '';
      
      if($currentPageIndex != $this->getTotalNumberofPages()-1)
      {
        $code .= '<li><a href="'.$urlToPostTo.'?paginationKey='.$this->paginationKey.'&pageIndex='.($currentPageIndex+1).'&'.$customQueryString.'" class="'.$class.' next" ajaxOutput="'.$ajaxOutput.'">></a></li>';
        $code .= '<li><a href="'.$urlToPostTo.'?paginationKey='.$this->paginationKey.'&pageIndex='.($this->getTotalNumberofPages()-1).'&'.$customQueryString.'" class="'.$class.' last" ajaxOutput="'.$ajaxOutput.'">>></a></li>';
      }
      else
      {
        $code .= '<li><a class="disabled">></a></li>';
        $code .= '<li><a class="disabled">>></a></li>';
      }

      return $code;
    }
    
    public function renderNav($currentPageIndex, $class='', $ajaxOutput='', $urlToPostTo='', $customQueryString='')
    {
      if($this->getTotalNumberofPages() < 2)
        return;
        
      $currentPageIndex = ($currentPageIndex < 0)? 0: $currentPageIndex;
      $currentPageIndex = ($currentPageIndex > ($this->getTotalNumberofPages()-1) )? $this->getTotalNumberofPages(): $currentPageIndex;
        
      $code = '<ul class="paging">';
      $code .= $this->renderFirstAndPrevious($currentPageIndex, $class, $ajaxOutput, $urlToPostTo, $customQueryString);


      $num = intval($this->maxPaginationTabs / 2);

      $start = $currentPageIndex-$num;
      $end = $currentPageIndex+$num;

      if($start < 0)
      {        
        $end += (0 - $start);
        $start = 0;
      }

      if($end > $this->getTotalNumberofPages())
      {
        $start -= ( $end - $this->getTotalNumberofPages() );
        $end = $this->getTotalNumberofPages();
      }
      $start = ( $start < 0 )? 0 :  $start;
      
      for($i=$start; $i < $end; $i++ )
      {
        $text = $i+1;
        
        if($i == $currentPageIndex)
          $code .='<li><a class="current">'.$text.'</a></li>';
        else
          $code .='<li><a href="'.$urlToPostTo.'?paginationKey='.$this->paginationKey.'&pageIndex='.$i.'&'.$customQueryString.'" class="'.$class.'" ajaxOutput="'.$ajaxOutput.'">'.$text.'</a></li>';
      }

      $code .= $this->renderNextAndLast($currentPageIndex, $class, $ajaxOutput, $urlToPostTo, $customQueryString);      
      $code .= '</ul><div class="clear"></div>';
      
      return $code;
    }
    
  }

?>
